import React, { useEffect, useRef, useState } from 'react';
import {
  AddDatabaseRequest,
  DatabaseType,
  databaseNameMapping,
  databaseProtocolMapping,
  useAppDispatch,
  useDatabaseAPI,
  useSchemaAPI,
  useSessionCache,
} from '@graphpolaris/shared/lib/data-access';
import { addError } from '@graphpolaris/shared/lib/data-access/store/configSlice';
import { ErrorOutline } from '@mui/icons-material';
import { mockDatabases } from './mockDatabases';
import { Dialog } from '@graphpolaris/shared/lib/components/Dialog';
import { Button } from '@graphpolaris/shared/lib/components/buttons';
import Input from '@graphpolaris/shared/lib/components/inputs';

const INITIAL_DB_STATE = {
  username: 'neo4j',
  password: 'DevOnlyPass',
  url: 'localhost',
  port: 7687,
  name: 'neo4j',
  protocol: 'neo4j://',
  internal_database_name: 'neo4j',
  type: DatabaseType.Neo4j,
};

interface Connection {
  connecting: boolean;
  status: null | string;
  verified: boolean | null;
}

export const NewDatabaseForm = (props: { onClose(): void; open: boolean }) => {
  const dispatch = useAppDispatch();
  const ref = useRef<HTMLDialogElement>(null);
  const [state, setState] = useState<AddDatabaseRequest>(INITIAL_DB_STATE);
  const api = useDatabaseAPI();
  const schemaApi = useSchemaAPI();
  const session = useSessionCache();
  const [hasError, setHasError] = useState({});
  const [sampleData, setSampleData] = useState<boolean | null>(false);
  const [connection, setConnection] = useState<Connection>({
    connecting: false,
    status: null,
    verified: null,
  });

  useEffect(() => {
    if (!state) return;
    if (state.type === DatabaseType.ArangoDB && state.port === 7687) {
      setState({ ...state, port: 8529 });
    } else if (state.type === DatabaseType.Neo4j && state.port === 8529) {
      setState({ ...state, port: 7687 });
    }
  }, [state.type]);

  function handleInputChange(field: string, value: unknown) {
    setState({
      ...state,
      [field]: value,
    });
  }

  async function testDatabaseConnection() {
    setConnection(() => ({
      connecting: true,
      status: 'Verifying database connection',
      verified: null,
    }));

    if (session.databases && !session.databases.hasOwnProperty(state.name)) {
      setConnection((prevState) => ({
        ...prevState,
        connecting: false,
        status: 'Database already connected',
        verified: false,
      }));
      return;
    }

    try {
      api
        .TestDatabaseConnection(state)
        .then((res: any) => {
          setConnection((prevState) => ({
            ...prevState,
            status: res.message,
            verified: res.verified,
          }));
        })
        .catch((error) => {
          setConnection((prevState) => ({
            ...prevState,
            connecting: false,
            status: 'Database connection failed',
            verified: false,
          }));
        });
    } catch (e) {
      setConnection((prevState) => ({
        ...prevState,
        status: 'Database connection failed',
        verified: false,
      }));
    }
  }

  function loadMockDatabase({ username, password, url, port, name, protocol, internal_database_name, type }: AddDatabaseRequest) {
    setState((prevState) => ({
      ...prevState,
      username,
      password,
      url,
      port,
      name,
      protocol,
      internal_database_name,
      type,
    }));
    setSampleData(false);
  }

  function handlePortChanged(port: string): void {
    if (!isNaN(Number(port))) setState({ ...state, port: Number(port) });
  }

  useEffect(() => {
    if (connection.verified === true) {
      try {
        api
          .AddDatabase(state, { updateDatabaseCache: true, setAsCurrent: true })
          .then(() => {
            schemaApi.RequestSchema(state.name);
          })
          .catch((e) => {
            dispatch(addError(e.message));
          });
      } catch (e: any) {
        dispatch(addError(e.message));
      } finally {
        closeDialog();
      }
    }
  }, [connection.verified]);

  function closeDialog(): void {
    setConnection({
      connecting: false,
      status: null,
      verified: null,
    });
    setState(INITIAL_DB_STATE);
    setSampleData(false);
    props.onClose();
    ref.current?.close();
  }

  return (
    <Dialog open={props.open} onClose={props.onClose} className="lg:min-w-[50rem] ">
      <div className="flex justify-between align-center">
        <h1 className="text-xl font-bold">New Database</h1>
        <div>
          {sampleData ? (
            <Button variant="outline" label="Go back" onClick={() => setSampleData(false)} />
          ) : (
            <>
              <h1 className="font-light text-xs">No data?</h1>
              <p className="font-light text-sm cursor-pointer underline" onClick={() => setSampleData(true)}>
                Try sample data
              </p>
            </>
          )}
        </div>
      </div>

      {sampleData ? (
        <div className="grid grid-cols-2 lg:grid-cols-3 gap-2">
          {mockDatabases.map((sample) => (
            <div
              key={sample.name}
              className="card hover:bg-secondary-100 cursor-pointer mb-2 border w-[15rem]"
              onClick={() => loadMockDatabase(sample)}
            >
              <div className="card-body">
                <h2 className="card-title">{sample.name}</h2>
                <p className="font-light text-secondary-400">{sample.subtitle}</p>
              </div>
            </div>
          ))}
        </div>
      ) : (
        <>
          <Input
            type="text"
            label="Name of the database"
            value={state.name}
            placeholder="neo4j"
            required
            errorText="This field is required"
            validate={(v) => {
              setHasError({ ...hasError, name: v.length === 0 });
              return v.length > 0;
            }}
            onChange={(value: string) => handleInputChange('name', value)}
          />

          <Input
            type="text"
            label="Internal database name"
            value={state.internal_database_name}
            placeholder="internal_database_name"
            required
            errorText="This field is required"
            validate={(v) => {
              setHasError({ ...hasError, internal_database_name: v.length === 0 });
              return v.length > 0;
            }}
            onChange={(value: string) => handleInputChange('internal_database_name', value)}
          />

          <div className="flex w-full gap-2">
            <Input
              type="dropdown"
              label="Database Type"
              required
              value={databaseNameMapping[state.type]}
              options={databaseNameMapping}
              onChange={(value: string | number) => {
                setState({
                  ...state,
                  type: databaseNameMapping.indexOf(String(value)),
                });
              }}
            />

            <Input
              type="dropdown"
              label="Database Protocol"
              required
              value={state.protocol}
              options={databaseProtocolMapping}
              onChange={(value: string | number) => {
                setState({
                  ...state,
                  protocol: String(value),
                });
              }}
            />
          </div>

          <div className="flex w-full gap-2">
            <Input
              type="text"
              label="Hostname/IP"
              value={state.url}
              placeholder="neo4j"
              required
              errorText="This field is required"
              validate={(v) => {
                setHasError({ ...hasError, url: v.length === 0 });
                return v.length > 0;
              }}
              onChange={(value: string) => handleInputChange('url', value)}
            />

            <Input
              type="text"
              label="Port"
              value={state.port.toString()}
              placeholder="neo4j"
              required
              errorText="Must be between 1 and 9999"
              validate={(v) => {
                setHasError({ ...hasError, port: !(v <= 9999 && v > 0) });
                return v <= 9999 && v > 0;
              }}
              onChange={(value: string) => handlePortChanged(value)}
            />
          </div>

          <div className="flex w-full gap-2">
            <Input
              type="text"
              label="Username"
              value={state.username}
              placeholder="username"
              required
              errorText="This field is required"
              validate={(v) => {
                setHasError({ ...hasError, username: v.length === 0 });
                return v.length > 0;
              }}
              onChange={(value: string) => handleInputChange('username', value)}
            />

            <Input
              type="text"
              label="Password"
              value={state.password}
              placeholder="password"
              required
              visible={false}
              errorText="This field is required"
              validate={(v) => {
                setHasError({ ...hasError, password: v.length === 0 });
                return v.length > 0;
              }}
              onChange={(value: string) => handleInputChange('password', value)}
            />
          </div>

          {!(connection.status === null) && (
            <div className={`flex flex-col justify-center items-center`}>
              <div className="flex justify-center items-center">
                {connection.verified === false && <ErrorOutline className="text-secondary-400" />}
                <p className="font-light text-sm	text-secondary-400	">{connection.status}</p>
              </div>
              {connection.verified === null && <progress className="progress w-56"></progress>}
            </div>
          )}
          <div className="grid gap-2">
            <Button
              type="primary"
              block
              label={connection.connecting ? 'Connecting...' : 'Connect'}
              onClick={(event) => {
                event.preventDefault();
                testDatabaseConnection();
              }}
              disabled={connection.connecting || Object.values(hasError).some((e) => e === true)}
            />
            <Button
              variant="outline"
              block
              label="Cancel"
              onClick={(event) => {
                event.preventDefault();
                closeDialog();
              }}
            />
          </div>
        </>
      )}
    </Dialog>
  );
};
