import React, { useEffect, useRef, useState } from 'react';
import {
  AddDatabaseRequest,
  databaseNameMapping,
  databaseProtocolMapping,
  useAppDispatch,
  useDatabaseAPI,
  DatabaseInfo,
} from '@graphpolaris/shared/lib/data-access';
import { ErrorOutline } from '@mui/icons-material';
import { Dialog } from '@graphpolaris/shared/lib/components/Dialog';
import { addError } from '@graphpolaris/shared/lib/data-access/store/configSlice';
import { Button } from '@graphpolaris/shared/lib/components/buttons';
import Input from '@graphpolaris/shared/lib/components/inputs';

interface Connection {
  updating: boolean;
  status: null | string;
  verified: boolean | null;
}

const DEFAULT_DB: AddDatabaseRequest = {
  name: '',
  internal_database_name: '',
  url: '',
  port: 7687,
  protocol: '',
  username: '',
  password: '',
  type: 0,
};

export const SettingsForm = (props: { onClose(): void; open: boolean; database: DatabaseInfo | null }) => {
  const dispatch = useAppDispatch();
  const ref = useRef<HTMLDialogElement>(null);
  const [state, setState] = useState<AddDatabaseRequest>(DEFAULT_DB);
  const api = useDatabaseAPI();
  const [hasError, setHasError] = useState({});
  const [connection, setConnection] = useState<Connection>({
    updating: false,
    status: null,
    verified: null,
  });

  useEffect(() => {
    const db = props.database;
    if (db) {
      setState({
        name: db.Name,
        internal_database_name: db.InternalDatabaseName,
        url: db.URL,
        port: db.Port,
        protocol: db.Protocol,
        username: db.Username,
        password: db.Password,
        type: db.Type,
      });
    }
  }, [props.database]);

  useEffect(() => {
    if (connection.verified) {
      api.GetAllDatabases().catch((e) => console.debug(e));
      closeDialog();
    }
  }, [connection.verified]);

  function handleInputChange(field: keyof AddDatabaseRequest, value: string) {
    if (field != 'port' && field != 'type') {
      setState((prevState) => ({ ...prevState, [field]: value }));
    }
  }

  async function handleSubmit() {
    setConnection(() => ({
      updating: true,
      status: 'Updating database credentials',
      verified: null,
    }));

    if (props.database != null) {
      try {
        api
          .UpdateDatabase({
            name: props.database.Name,
            internal_database_name: state.internal_database_name,
            password: state.password,
            port: state.port,
            protocol: state.protocol,
            type: state.type,
            url: state.url,
            username: state.username,
          })
          .then(() => {
            setConnection(() => ({
              updating: false,
              status: 'Worked',
              verified: true,
            }));
          })
          .catch((e) => {
            dispatch(addError('Database updating went wrong'));
            setConnection((prevState) => ({
              ...prevState,
              updating: false,
              status: 'Database updating went wrong',
              verified: false,
            }));
          });
      } catch (e) {
        dispatch(addError('An error occured'));
        setConnection((prevState) => ({
          updating: false,
          status: 'An error occured',
          verified: false,
        }));
      }
    }
  }

  function handlePortChanged(port: string): void {
    if (!isNaN(Number(port))) setState({ ...state, port: Number(port) });
  }

  function closeDialog(): void {
    setConnection({
      updating: false,
      status: null,
      verified: null,
    });
    setState(DEFAULT_DB);
    props.onClose();
    ref.current?.close();
  }

  return (
    <Dialog open={props.open} onClose={props.onClose}>
      <div className="flex justify-between align-center">
        <h1 className="card-title">Update {state.name} Database</h1>
      </div>

      <>
        <Input type="text" label="Name of database" value={state.name} onChange={() => {}} disabled />

        <Input
          type="text"
          label="Internal database name"
          value={state.internal_database_name}
          placeholder="internal_database_name"
          required
          errorText="This field is required"
          validate={(v) => {
            setHasError({ ...hasError, internal_database_name: v.length === 0 });
            return v.length > 0;
          }}
          onChange={(value: string) => handleInputChange('internal_database_name', value)}
        />

        <div className="flex w-full gap-2">
          <Input
            type="dropdown"
            label="Database Type"
            required
            value={databaseNameMapping[state.type]}
            options={databaseNameMapping}
            onChange={(value: string | number) => {
              setState({
                ...state,
                type: databaseNameMapping.indexOf(String(value)),
              });
            }}
          />

          <Input
            type="dropdown"
            label="Database Protocol"
            required
            value={state.protocol}
            options={databaseProtocolMapping}
            onChange={(value: string | number) => {
              setState({
                ...state,
                protocol: String(value),
              });
            }}
          />
        </div>

        <div className="flex w-full gap-2">
          <Input
            type="text"
            label="Hostname/IP"
            value={state.url}
            placeholder="neo4j"
            required
            errorText="This field is required"
            validate={(v) => {
              setHasError({ ...hasError, url: v.length === 0 });
              return v.length > 0;
            }}
            onChange={(value: string) => handleInputChange('url', value)}
          />

          <Input
            type="text"
            label="Port"
            value={state.port.toString()}
            placeholder="neo4j"
            required
            errorText="Must be between 1 and 9999"
            validate={(v) => {
              setHasError({ ...hasError, port: !(v <= 9999 && v > 0) });
              return v <= 9999 && v > 0;
            }}
            onChange={(value: string) => handlePortChanged(value)}
          />
        </div>

        <div className="flex w-full gap-2">
          <Input
            type="text"
            label="Username"
            value={state.username}
            placeholder="username"
            required
            errorText="This field is required"
            validate={(v) => {
              setHasError({ ...hasError, username: v.length === 0 });
              return v.length > 0;
            }}
            onChange={(value: string) => handleInputChange('username', value)}
          />

          <Input
            type="text"
            label="Password"
            value={state.password}
            visible={false}
            placeholder="password"
            required
            errorText="This field is required"
            validate={(v) => {
              setHasError({ ...hasError, password: v.length === 0 });
              return v.length > 0;
            }}
            onChange={(value: string) => handleInputChange('password', value)}
          />
        </div>

        {!(connection.status === null) && (
          <div className={`flex flex-col justify-center items-center`}>
            <div className="flex justify-center items-center">
              {connection.verified === false && <ErrorOutline className="text-secondary-400" />}
              <p className="font-light text-sm	text-secondary-400	">{connection.status}</p>
            </div>
            {connection.verified === null && <progress className="progress w-56"></progress>}
          </div>
        )}

        <div className="grid md:grid-cols-2 gap-3 card-actions w-full justify-stretch items-center">
          <Button
            type="primary"
            label={connection.updating ? 'Updating...' : 'Update'}
            onClick={(event) => {
              event.preventDefault();
              handleSubmit();
            }}
            disabled={connection.updating || Object.values(hasError).some((e) => e === true)}
          />
          <Button
            variant="outline"
            label="Cancel"
            onClick={(event) => {
              event.preventDefault();
              closeDialog();
            }}
          />
        </div>
      </>
    </Dialog>
  );
};
